// ScrollDialog.cpp : implementation file
//

#include "stdafx.h"
#include "ScrollDialog.h"

// CScrollDialog

// The following definitions specify how many points 
// are scrolled at each press of the scrollbar arrows
#define HORZ_PTS 8
#define VERT_PTS 8

IMPLEMENT_DYNAMIC(CScrollDialog, CDialogBase)

CScrollDialog::CScrollDialog(UINT ID, CWnd* pParent /*NULL*/)
	: CDialogBase(ID, pParent)
{
	m_rcOrg.SetRectEmpty();
	m_nHscrollPos = m_nVscrollPos = 0;
	m_nHscrollMax = m_nVscrollMax = 0;
	m_nHorzInc = m_nVertInc = 0;
}

CScrollDialog::~CScrollDialog()
{
}


BEGIN_MESSAGE_MAP(CScrollDialog, CDialogBase)
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_MOUSEWHEEL()
END_MESSAGE_MAP()


void CScrollDialog::InitRect(CRect rect)
{
	m_rcOrg = rect;
}

void CScrollDialog::ResetScrollbars()
{
	// Reset our window scrolling information
	ScrollWindow(m_nHscrollPos * HORZ_PTS, 0, NULL, NULL);
	ScrollWindow(0, m_nVscrollPos * VERT_PTS, NULL, NULL);
	m_nHscrollPos = 0;
	m_nVscrollPos = 0;
	SetScrollPos(SB_HORZ, m_nHscrollPos, TRUE);
	SetScrollPos(SB_VERT, m_nVscrollPos, TRUE);
}

void CScrollDialog::SetupScrollbars()
{
	CRect tempRect;				
	GetClientRect(&tempRect);	//  ۾  ũ Ի.

	WINDOWPLACEMENT wp;			//     մ .
	GetWindowPlacement(&wp);	//   .
	BOOL bMaximized = (wp.showCmd == SW_MAXIMIZE);	// ̾ α ִ ũ .

	// Calculate how many scrolling increments for the client area
	m_nHorzInc = (m_rcOrg.Width() - tempRect.Width()) / HORZ_PTS;
	m_nVertInc = (m_rcOrg.Height() - tempRect.Height()) / VERT_PTS;

	// Set the vertical and horizontal scrolling info
	m_nHscrollMax = max(0, m_nHorzInc);
	m_nHscrollPos = min (m_nHscrollPos, m_nHscrollMax);
	SetScrollRange(SB_HORZ, 0, m_nHscrollMax, FALSE);
	SetScrollPos(SB_HORZ, m_nHscrollPos, TRUE);
	m_nVscrollMax = max(0, m_nVertInc);
	m_nVscrollPos = min(m_nVscrollPos, m_nVscrollMax);
	SetScrollRange(SB_VERT, 0, m_nVscrollMax, FALSE);
	SetScrollPos(SB_VERT, m_nVscrollPos, TRUE);
}

// CScrollDialog message handlers

BOOL CScrollDialog::OnInitDialog()
{
	CDialogBase::OnInitDialog();

	m_nHscrollPos = 0;
	m_nVscrollPos = 0;

	CRect rect;
	GetClientRect(&rect);
	InitRect(rect);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CScrollDialog::OnSize(UINT nType, int cx, int cy)
{
	CDialogBase::OnSize(nType, cx, cy);

	if (m_rcOrg.IsRectEmpty() == FALSE) {
		ResetScrollbars();
		SetupScrollbars();
	}
}

void CScrollDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	// Handle horizontal scrollbar messages
	// These can be tweaked to better fit the implementation
	int nInc;
	switch (nSBCode) {
	case SB_TOP: nInc = -m_nHscrollPos; break;
	case SB_BOTTOM: nInc = m_nHscrollMax-m_nHscrollPos; break;
	case SB_LINEUP: nInc = -1; break;
	case SB_LINEDOWN: nInc = 1; break;
	case SB_PAGEUP: nInc = -HORZ_PTS; break;
	case SB_PAGEDOWN: nInc = HORZ_PTS; break;
	case SB_THUMBTRACK: nInc = nPos - m_nHscrollPos; break;
	default: nInc = 0;
	}

	nInc = max(-m_nHscrollPos, min(nInc, m_nHscrollMax - m_nHscrollPos));

	if (nInc) {
		m_nHscrollPos += nInc;
		int iMove = -HORZ_PTS * nInc;
		ScrollWindow(iMove, 0, NULL, NULL);
		SetScrollPos(SB_HORZ, m_nHscrollPos, TRUE);
	}

	CDialogBase::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CScrollDialog::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	// Handle vertical scrollbar messages
	// These can be tweaked to better fit the implementation
	int nInc;
	switch (nSBCode) {
	case SB_TOP: nInc = -m_nVscrollPos; break;
	case SB_BOTTOM: nInc = m_nVscrollMax-m_nVscrollPos; break;
	case SB_LINEUP: nInc = -1; break;
	case SB_LINEDOWN: nInc = 1; break;
	case SB_PAGEUP: nInc = min(-1, -m_nVertInc); break;
	case SB_PAGEDOWN: nInc = max(1, m_nVertInc); break;
	case SB_THUMBTRACK: nInc = nPos - m_nVscrollPos; break;
	default: nInc = 0;
	}

	nInc = max(-m_nVscrollPos, min(nInc, m_nVscrollMax - m_nVscrollPos));

	if (nInc) {
		m_nVscrollPos += nInc;
		int iMove = -VERT_PTS * nInc;
		ScrollWindow(0, iMove, NULL, NULL);
		SetScrollPos(SB_VERT, m_nVscrollPos, TRUE);
	}

	CDialogBase::OnVScroll(nSBCode, nPos, pScrollBar);
}

BOOL CScrollDialog::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	int nMin, nMax;
	GetScrollRange(SB_VERT, &nMin, &nMax);

	if (nMax > nMin) {
		int nInc=0; 
		if (zDelta==120) { //  콺  
			nInc = -1;
		}
		else if (zDelta==-120) { // Ʒ 콺  
			nInc = 1;
		}

		nInc = max(-m_nVscrollPos, min(nInc, m_nVscrollMax - m_nVscrollPos));

		if (nInc != 0){
			m_nVscrollPos += nInc;
			int iMove = -VERT_PTS * nInc;
			ScrollWindow(0, iMove, NULL, NULL);
			SetScrollPos(SB_VERT, m_nVscrollPos, TRUE);
		}	
	}

	return CDialogBase::OnMouseWheel(nFlags, zDelta, pt);
}
